module PackageService
  class Generator
    def initialize(objects)
      @objects = objects.is_a?(Array) ? objects : [objects]
      @cache_key = nil

      raise 'Invalid objects for PackageService' unless valid?
    end

    def generate
      Rails.cache.write(PackageService.job_key(cache_key), 'wait')
      PackageWorker.perform_async(objects_map, filename, cache_key)
    end

    def cache_key
      return @cache_key unless @cache_key.nil?

      key_name = 'package_service'
      cache_list = []

      @objects.sort_by!(&:id)
      @objects.each { |o| cache_list << o.cache_key }

      @cache_key = "#{key_name}[#{cache_list.join(',')}]"
    end

    private

    def valid?
      allowed_classes = PackageService.allowed_classes

      @objects.each do |object|
        return false unless allowed_classes.include? object.class.name
      end

      true
    end

    def filename
      loop do
        name = "#{SecureRandom.hex(12)}.zip"
        return name unless File.exist?(PackageService.file_root(name))
      end
    end

    def objects_map
      @objects.map { |o| { class: o.class, id: o.id } }
    end
  end
end
